@@ -1,11 +1,7 @@
 package Mail::Milter::Authentication::Exception;
-use 5.20.0;
 use strict;
 use warnings;
-use Mail::Milter::Authentication::Pragmas;
-# ABSTRACT: Class representing an exception
-our $VERSION = '2.20210105.2'; # VERSION
-
+our $VERSION = '2.20180510'; # VERSION
 
 sub new {
     my ( $class, $args ) = @_;
@@ -24,19 +20,11 @@ __END__
 
 =head1 NAME
 
-Mail::Milter::Authentication::Exception - Class representing an exception
+Mail::Milter::Authentication::Exception
 
 =head1 VERSION
 
-version 2.20210105.2
-
-=head1 CONSTRUCTOR
-
-=head2 I<new( $args )>
-
-die Mail::Milter::Authentication::Exception->new({ 'Type' => 'Timeout', 'Text' => 'Example timeout exception' });
-
-Create a new exception object.
+version 2.20180510
 
 =head1 AUTHOR
 
@@ -44,7 +32,7 @@ Marc Bradshaw <marc@marcbradshaw.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Marc Bradshaw.
+This software is copyright (c) 2018 by Marc Bradshaw.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
