@@ -2,7 +2,7 @@ package Mail::Milter::Authentication::Handler::AlignedFrom;
 use strict;
 use warnings;
 use base 'Mail::Milter::Authentication::Handler';
-our $VERSION = '2.20180509'; # VERSION
+our $VERSION = '2.20180510'; # VERSION
 
 use Net::DNS;
 use Sys::Syslog qw{:standard :macros};
@@ -146,9 +146,9 @@ sub eom_callback {
             my $dmarc_handler = $self->get_handler('DMARC');
             my $dmarc_object = $dmarc_handler->get_dmarc_object();
             my $org_smtp_domain   = eval{ $dmarc_object->get_organizational_domain( $self->{ 'smtp_domain' } ); };
-            if ( my $error = $@ ) { die $error if $error =~ /Timeout/; }
+            $self->handle_exception( $@ );
             my $org_header_domain = eval{ $dmarc_object->get_organizational_domain( $self->{ 'header_domain' } ); };
-            if ( my $error = $@ ) { die $error if $error =~ /Timeout/; }
+            $self->handle_exception( $@ );
 
             if ( $org_smtp_domain eq $org_header_domain ) {
                 $result = 'orgdomain_pass';
@@ -193,7 +193,7 @@ Mail::Milter::Authentication::Handler::AlignedFrom
 
 =head1 VERSION
 
-version 2.20180509
+version 2.20180510
 
 =head1 DESCRIPTION
 
