@@ -2,7 +2,7 @@ package Mail::Milter::Authentication::Handler::DKIM;
 use strict;
 use warnings;
 use base 'Mail::Milter::Authentication::Handler';
-our $VERSION = '2.20180509'; # VERSION
+our $VERSION = '2.20180510'; # VERSION
 
 use Data::Dumper;
 use English qw{ -no_match_vars };
@@ -105,9 +105,9 @@ sub eoh_callback {
             $self->set_object('dkim', $dkim, 1);
         };
         if ( my $error = $@ ) {
+            $self->handle_exception( $error );
             $self->log_error( 'DKIM Setup Error ' . $error );
             $self->{'failmode'} = 1;
-            die $error if $error =~ /Timeout/;
             $self->_check_error( $error );
             $self->metric_count( 'dkim_total', { 'result' => 'error' } );
             delete $self->{'headers'};
@@ -121,9 +121,9 @@ sub eoh_callback {
             );
         };
         if ( my $error = $@ ) {
+            $self->handle_exception( $error );
             $self->log_error( 'DKIM Headers Error ' . $error );
             $self->{'failmode'} = 1;
-            die $error if $error =~ /Timeout/;
             $self->_check_error( $error );
             $self->metric_count( 'dkim_total', { 'result' => 'error' } );
         }
@@ -163,9 +163,9 @@ sub body_callback {
         $dkim->PRINT( $dkim_chunk );
     };
     if ( my $error = $@ ) {
+        $self->handle_exception( $error );
         $self->log_error( 'DKIM Body Error ' . $error );
         $self->{'failmode'} = 1;
-        die $error if $error =~ /Timeout/;
         $self->_check_error( $error );
         $self->metric_count( 'dkim_total', { 'result' => 'error' } );
     }
@@ -347,11 +347,11 @@ sub eom_callback {
         }
     };
     if ( my $error = $@ ) {
+        $self->handle_exception( $error );
 
         # Also in DMARC module
         $self->log_error( 'DKIM EOM Error ' . $error );
         $self->{'failmode'} = 1;
-        die $error if $error =~ /Timeout/;
         $self->_check_error( $error );
         $self->metric_count( 'dkim_total', { 'result' => 'error' } );
         return;
@@ -422,7 +422,7 @@ Mail::Milter::Authentication::Handler::DKIM
 
 =head1 VERSION
 
-version 2.20180509
+version 2.20180510
 
 =head1 DESCRIPTION
 
