@@ -2,7 +2,7 @@ package Mail::Milter::Authentication::Handler::SPF;
 use strict;
 use warnings;
 use base 'Mail::Milter::Authentication::Handler';
-our $VERSION = '2.20180509'; # VERSION
+our $VERSION = '2.20180510'; # VERSION
 
 use Sys::Syslog qw{:standard :macros};
 use Mail::AuthenticationResults::Header::Entry;
@@ -42,8 +42,8 @@ sub setup_callback {
             );
         };
         if ( my $error = $@ ) {
+            $self->handle_exception( $error );
             $self->log_error( 'SPF Object Setup Error ' . $error );
-            die $error if $error =~ /Timeout/;
         }
         $thischild->{'object'}->{$name} = {
             'object'  => $object,
@@ -186,9 +186,9 @@ sub envfrom_callback {
         }
     };
     if ( my $error = $@ ) {
+        $self->handle_exception( $error );
         $self->log_error( 'SPF Error ' . $error );
         $self->{'failmode'} = 1;
-        die $error if $error =~ /Timeout/;
         my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'spf' )->safe_set_value( 'temperror' );
         $self->add_auth_header($header);
         $self->metric_count( 'spf_total', { 'result' => 'error' } );
@@ -221,7 +221,7 @@ Mail::Milter::Authentication::Handler::SPF
 
 =head1 VERSION
 
-version 2.20180509
+version 2.20180510
 
 =head1 DESCRIPTION
 
