@@ -2,7 +2,7 @@ package Mail::Milter::Authentication::Handler::SenderID;
 use strict;
 use warnings;
 use base 'Mail::Milter::Authentication::Handler';
-our $VERSION = '2.20180509'; # VERSION
+our $VERSION = '2.20180510'; # VERSION
 
 use Sys::Syslog qw{:standard :macros};
 use Mail::AuthenticationResults::Header::Entry;
@@ -120,8 +120,8 @@ sub eoh_callback {
         }
     };
     if ( my $error = $@ ) {
+        $self->handle_exception( $error );
         $self->log_error( 'SENDERID Error ' . $error );
-        die $error if $error =~ /Timeout/;
         $self->metric_count( 'senderid_total', { 'result' => 'error' } );
         my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'senderid' )->safe_set_value( 'temperror' );
         $self->add_auth_header($header);
@@ -151,7 +151,7 @@ Mail::Milter::Authentication::Handler::SenderID
 
 =head1 VERSION
 
-version 2.20180509
+version 2.20180510
 
 =head1 DESCRIPTION
 
